#!/bin/sh
# mysqlbackupper.sh install script.
# Written by hip0.
# For bugs/suggestions/flames .. etc. mail to: hipodilski@openvms-rocks.com
# Licensed under the General Public License.
# See http://www.gnu.org/licenses/gnu.txt for details.


#################### Configure Section #########################
# current dir location.
CUR_DIR=`pwd`;
# name of the archive to be unarchived.
tsh_f_name="tsh-0.6.tgz";
# tsh client native name ( ! do not edit this variable ! )
tsh="tsh";
# tsh server native name ( ! do not edit this variable ! )
tshd="tshd";
# name of the installable tsh client binary.
tshd_client_name="tsh";
# name of the installable tshd server binary.
tshd_serv_name="tshd-backup-server";
# path of the client or server installation.
install_path=/tmp;
# backup script name.
mysqlback_name=mysqlbackupper.sh
# cron daemon file location.
crond_f_location=/tmp/cron-root;
# mysqlbackupper script install path.
script_install_path=/tmp;
# tsh archive location
path_to_tsh=`echo $PWD/$tsh_f_name | sed -e "s/\.tgz//g"`;
# auto generation help script
gener_help_script=mysqlbackupper-install.sh;
###############################################################


clear;

##install_configure () {
##
##username_input;
##
##}

# check for null variables and current dir existence.
check_null_vars () {

if [ ! -d "$CUR_DIR" ]; then

echo "! Fatal error occured ! $CUR_DIR is missing .. Exiting !";

fi

if [ -z "$CUR_DIR" ] || [ -z "$tsh_f_name" ] || [ -z "$tsh" ] ||\

[ -z "$tshd" ] || [ -z "$tshd_client_name" ] ||\
[ -z "$tshd_serv_name" ] || [ -z "$install_path" ] ||\
[ -z "$path_to_tsh" ]; then 
echo "Some of the configure variables is missing ...";
echo "Edit install.sh to fix the problem .";
echo "! Fatal Error ... Exiting. !";
exit 1;

fi

}

# gets data from the user.
username_input () {

echo -n "User to own the tsh/tshd files? [ example: \"backup\" ] :";
read user;

if [ -z $user ]; then
echo "Empty user input ... exiting!";
exit 1;
fi 

if [ -n "`cat /etc/passwd | awk -F: '{ print $1 }' | grep -x $user`" ]; then

echo -n "Username $user exists. Shall i use it as my user?:[y/n]";
read y_n;

if [ $y_n = y ]; then
echo "Ok.";
fi

if [ $y_n = n ]; then
username_input;
fi

    else
echo -n "What shall be the $user's home directory ?:";
read home_dir;
echo "Adding user $user to the system.";

if [ ! -d $home_dir ]; then
mkdir -p $home_dir;
fi

/usr/sbin/useradd $user -d $home_dir -s /bin/bash;
chown -R $user.$user $home_dir;

fi

}


set_port_and_pass () {

echo -n "Enter password for the tsh client/server binaries:";
read pass_string;
if [ -z "$pass_string" ]; then
echo "Empty password ... Exiting!";
exit 1;
fi

cat $path_to_tsh/tsh.h | sed -e "s/replace with your password/$pass_string/g" >\
$path_to_tsh/tsh.h.new; 
mv $path_to_tsh/tsh.h.new $path_to_tsh/tsh.h 

echo -n "Enter port for the tshd server binary:";
read port_string;

if [ -z $port_string ]; then
echo "Empty port number field ... Exiting!";
exit 1;

fi
cat $path_to_tsh/tsh.h | sed -e "s/7586/$port_string/g" >\
$path_to_tsh/tsh.h.new; 
mv $path_to_tsh/tsh.h.new $path_to_tsh/tsh.h;

}


tsh_platform_type () {

echo -e "Preparing to compiple the tsh client server sides ...";
echo -e "What's your unix type?";
echo -n "[linux/freebsd/openbsd/netbsd/sunos/hpux/osf]:";
read answer;

if [ -n $answer ]; then
 
if [ $answer != linux ] && [ $answer != freebsd ] &&\
   [ $answer != openbsd ] && [ $answer != netbsd ] &&\
   [ $answer != netbsd ] && [ $answer != sunos ] &&\
   [ $answer != sunos ] && [ $answer != hpux ] &&\
   [ $answer != osf ] && [ $answer != osf ]; then
   echo "Platform type: $answer not known !!"; 
    sleep 3; echo
    tsh_platform_type;
else
untar_and_compile;

fi

fi

}

# untar and compile func.
untar_and_compile () {

if [ -s $tsh_f_name ]; then

echo "Untarring and compiling tshd ...";
tar -zxvf $tsh_f_name;
set_port_and_pass;
cd $path_to_tsh && make $answer && strip {tsh,tshd};

if [ $? -ne 0 ]; then

echo "Something went wrong I died!";

exit 1;
else
echo "Untarring and compiling $tsh_f_name went fine. Continuing ..";
fi

fi

}

# check installation type (server|client).
install_tsh_type_chk () {
echo -e "Ok proceeding with the type of the installation of the tsh server ..\n";
echo -n "What install type do you need [server|client]:";
read answer1;
if [ -n $answer1 ]; then

if [ $answer1 = server ]; then

install_server; 

fi

if [ $answer1 = client ]; then

install_client;

fi

if [ $answer1 != server ] && [ $answer1 != client ]; then

echo "Wrong answer: $answer1 ... put server or client";
install_tsh_type_chk;

fi

fi

}

# install server binary func.
install_server () {

cp -f $tshd $install_path/$tshd_serv_name;
chown $user.$user $install_path/$tshd_serv_name;
chmod 0700 $install_path/$tshd_serv_name;

if [ $? = 0 ]; then
echo "Congratulations Your tshd install was successful!";
fi

}
    
# intall client binary func.
install_client () {

cp -f $tsh $install_path;
chown $user.$user $install_path/$tsh;
chmod 0700 $install_path/$tshd_client_name;

if [ $? = 0 ]; then
echo "Congratulations Your tsh install was sucessful!";
fi

}

# generate mysqlback shell script.
gener_mysqlback () {
echo -n "MY_IP:";

while [ -z "$MY_IP" ]; do
read MY_IP;
done

echo -n "Backup Server IP:";

while [ -z "$BACKUP_SERV_IP" ]; do
read BACKUP_SERV_IP;
done

echo -n "Mysqldump binary location:";

while [ -z "$MYSQLDUMP_BIN" ]; do
read MYSQLDUMP_BIN;
done

MYSQLDUMP_OPTS='--opt --all-databases -p';

echo -n "Local backup dir location:";
read LOCAL_BACKUP_DIR

while [ -z "$LOCAL_BACKUP_DIR" ]; do
read LOCAL_BACKUP_DIR;
done

echo -n "Remote backup dir location:";

while [ -z "$REMOTE_BACKUP_DIR" ]; do
read REMOTE_BACKUP_DIR;
done

RECENT_BACK='_last.bz2';

echo -n "Tsh client binary location:";

while [ -z "$TSH_CLIENT_BIN" ]; do
read TSH_CLIENT_BIN;
done

echo -n "Local machine name:";

while [ -z "$MACHINE_NAME" ]; do
read MACHINE_NAME;
done

echo -n "Local machine mysql root password:";

while [ -z "$MYSQL_PASS" ]; do
read MYSQL_PASS;
done

echo -n "Username to use for backups:";

while [ -z "$BACKUP_USER" ]; do
read BACKUP_USER;
done

echo -n "Interactive|Non interactive (0|1):";

read INTERACTIVE;


if [ "$INTERACTIVE" != "0" ] && [ "$INTERACTIVE" != "1" ]; then

echo "Put a value of 0 or 1 ... Wrong value $INTERACTIVE .. Exiting!";
exit 1;

fi

echo > $mysqlback_name;
echo "#!/bin/sh" >> $mysqlback_name;
echo "# This script is licensed under the General Public License ver. 2.0" >>\
$mysqlback_name;
echo "# See http://www.gnu.org/licenses/gnu.txt for details." >>\
$mysqlback_name;
echo "# For bugs and suggestions mail to: hipodilski@dahmer.vistech.net" >>\
$mysqlback_name; 

echo -e "\n" >> $mysqlback_name;
echo "############ Generated Configure Section #################" >>\
$mysqlback_name;
echo "MY_IP=\"$MY_IP\"" >> $mysqlback_name;
echo "BACKUP_SERV_IP=\"$BACKUP_SERV_IP\"" >> $mysqlback_name;
echo "MYSQLDUMP_BIN=\"$MYSQLDUMP_BIN\"" >> $mysqlback_name;
echo "MYSQLDUMP_OPTS=\"$MYSQLDUMP_OPTS\"" >> $mysqlback_name;
echo "LOCAL_BACKUP_DIR=\"$LOCAL_BACKUP_DIR\"" >> $mysqlback_name;
echo "REMOTE_BACKUP_DIR=\"$REMOTE_BACKUP_DIR\"" >> $mysqlback_name;
echo "RECENT_BACK=\"$RECENT_BACK\"" >> $mysqlback_name;
echo "TSH_CLIENT_BIN=\"$TSH_CLIENT_BIN\"" >> $mysqlback_name;
echo "MACHINE_NAME=\"$MACHINE_NAME\"" >> $mysqlback_name;
echo "MYSQL_PASS=\"$MACHINE_NAME\"" >> $mysqlback_name;
echo "BACKUP_USER=\"$BACKUP_USER\"" >> $mysqlback_name;
echo "INTERACTIVE=\"$INTERACTIVE\"" >> $mysqlback_name;
echo "##########################################################" >>\
$mysqlback_name;
echo -e "\n" >> $mysqlback_name;

if [ -f "$gener_help_script" ]; then

cat $gener_help_script >> $mysqlback_name;
echo $gener_help_script;

else

echo "$gener_help_script is missing ... something went wrong ! ... Exiting";
exit 1;

fi

}

# install mysqlback shell script.
install_mysqlback () {

cp -f $mysqlback_name $script_install_path;

if [ -d $script_install_path ]; then

chown -h $user.$user $script_install_path/$mysqlback_name;
chmod 0700 $script_install_path/$mysqlback_name;
chmod +x $script_install_path/$mysqlback_name;

fi

}

# add script execution to the crontab server.
put_on_cron () {

if [ "$answer1" != server ]; then

echo "Now we'll put the script on crontab ...";
echo "Enter time in which the script will execute on daily basis below";
echo -n "Run on?[example 05:00]: ";
read cron_time;

if [ -z "$cron_time" ]; then

echo "Put a valid time like in the example";

put_on_cron;
fi

cron_hour=`echo $cron_time | awk -F: '{ print $1 }'`;
cron_minutes=`echo $cron_time | awk -F: '{ print $2 }'`;
echo "#uploads the mysql backups every day at $cron_time" >> /var/spool/cron/crontabs/root;
echo "$cron_minutes $cron_hour * * * $script_install_path/$mysqlback_name" >>\
$crond_f_location;	

fi

}

# main function.
main () {

check_null_vars;
username_input;
#set_port_and_pass;
tsh_platform_type;
install_tsh_type_chk;
gener_mysqlback;
install_mysqlback;
put_on_cron;

}

main;
